@@ -27,7 +27,6 @@ model <- crs(y~x1+x2,
              cv=cv,
              complexity="degree-knots",
              knots="uniform",
-             deriv=1,
              cv.func="cv.aic",
              nmulti=nmulti)
 
@@ -60,7 +59,7 @@ par3d(windowRect=c(900,100,900+640,100+640))
 rgl.viewpoint(theta = 0, phi = -70, fov = 80)
 
 persp3d(x=x1.seq,y=x2.seq,z=z,
-        xlab="x1",ylab="x2",zlab="y",
+        xlab="X1",ylab="X2",zlab="Y",
         ticktype="detailed",      
         border="red",
         color=col,
@@ -83,4 +82,6 @@ grid3d(c("x", "y+", "z"))
 ## Note - to plot an rgl figure first get it oriented how you want
 ## (i.e. resize, rotate etc.) and then call rgl.postscript to create,
 ## i.e., a PDF of your graphic as in
-## rgl.postscript("persp3db.pdf","pdf")
+## rgl.postscript("foo.pdf","pdf"). Or better still,
+## rgl.snapshot("foo.png") for a png that can be called directly in
+## LaTeX via \includegraphics[scale=.6]{foo.png}
